-------------------------------------------------------------------------
-- (C) COPYRIGHT International Business Machines Corp. 2000-2003
-- All Rights Reserved
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
-- dsyctldb.ddl; DB2 Everyplace Mobile Devices Control Database SQL.
-------------------------------------------------------------------------

--* Update database DSYCTLDB performance parameters
update db config for dsyctldb using applheapsz 1024;
update db config for dsyctldb using logfilsiz 5120;
update db config for dsyctldb using logprimary 6;
update db config for dsyctldb using logsecond 4;

--* Create schema DSY (if does not exist)
create schema DSY;

--* Create table DSY.GROUPS (if does not exist)
create table
  DSY.GROUPS(
    ID char(13) for bit data primary key not null,
    NAME varchar(254) unique not null,
    DESCRIPTION varchar(128),
    ENABLED char(1) not null with default 'N',
    LDAP_UID varchar(254),
    LDAP_SECURITYNAME varchar(254),
    DATA_LEVEL int not null with default 0,
    constraint CK_ENABLED check(ENABLED in('N','Y'))
  );
alter table DSY.GROUPS add column LDAP_UID varchar(254);
alter table DSY.GROUPS add column LDAP_SECURITYNAME varchar(254);
alter table DSY.GROUPS add column DATA_LEVEL int not null with default 0;
alter table DSY.GROUPS alter column NAME set data type varchar(254);

--* Create table DSY.CLIENTS (if does not exist)
create table
  DSY.CLIENTS(
    GROUP_ID char(13) for bit data,
    ID char(13) for bit data primary key not null,
    NAME varchar(254) unique not null,
    DESCRIPTION varchar(128),
    PASSWORD varchar(254),
    NEXTNONCE varchar(16),
    LDAP_UID varchar(254),
    LDAP_SECURITYNAME varchar(254),
    DB2E_USERID varchar(18),
    DB2E_PASSWORD varchar(18),
    constraint FK_GROUP_ID foreign key(GROUP_ID) references DSY.GROUPS(ID) on delete set null
  );
alter table DSY.CLIENTS add column NEXTNONCE varchar(16);
alter table DSY.CLIENTS add column LDAP_UID varchar(254);
alter table DSY.CLIENTS add column LDAP_SECURITYNAME varchar(254);
alter table DSY.CLIENTS add column DB2E_USERID varchar(18);
alter table DSY.CLIENTS add column DB2E_PASSWORD varchar(18);
alter table DSY.CLIENTS alter column NAME set data type varchar(254);
alter table DSY.CLIENTS alter column PASSWORD set data type varchar(254);

create index DSY.CLIENT_GROUP_ID on DSY.CLIENTS(GROUP_ID);

CREATE TABLE DSY.SESSIONMONITOR  (
		  CLIENT_NAME VARCHAR(254) NOT NULL , 
		  DEVICE_ID VARCHAR(56) NOT NULL , 
		  SUBSCRIPTION_ID VARCHAR(26) NOT NULL , 
		  SESSIONNUM INTEGER , 
		  STATE INTEGER );   

-- DDL Statements for primary key on Table DSY.SESSIONMONITOR
ALTER TABLE DSY.SESSIONMONITOR ADD PRIMARY KEY (DEVICE_ID, SUBSCRIPTION_ID);
ALTER TABLE DSY.SESSIONMONITOR VOLATILE;

-- Early v8.1.4 entries may have a NULL state value when all done.
-- This will translate those rows to the appropriate state value of "8" for all done.
UPDATE DSY.SESSIONMONITOR SET STATE=8 WHERE STATE IS NULL;

--* Create table DSY.DEVICES (if does not exist)
create table
  DSY.DEVICES(
    CLIENT_ID char(13) for bit data not null,
    ID varchar(56) not null,
    TYPE varchar(4) not null,
    ENABLED char(1) not null with default 'N',
    VERSION varchar(18),
    TIMESTAMP timestamp with default current timestamp,
    CONFIG_LEVEL integer not null with default -1,
    DATA_LEVEL integer not null with default -1,
    DB2E_USERID varchar(18),
    DB2E_PASSWORD varchar(18),
    constraint FK_CLIENT_ID foreign key(CLIENT_ID) references DSY.CLIENTS(ID) on delete cascade,
    constraint UN_DEVICE_ID unique(ID),
    constraint CK_ENABLED check(ENABLED in('N','Y'))
  );
alter table DSY.DEVICES add column VERSION varchar(18);
alter table DSY.DEVICES alter column ID set data type varchar(56);
alter table DSY.DEVICES add column TIMESTAMP timestamp with default current timestamp;
alter table DSY.DEVICES add column CONFIG_LEVEL integer not null with default -1;
alter table DSY.DEVICES add column DATA_LEVEL integer not null with default -1;
alter table DSY.DEVICES add column DB2E_USERID varchar(18);
alter table DSY.DEVICES add column DB2E_PASSWORD varchar(18);
drop table DSY.DEVICECONFIG;
alter table DSY.DEVICES drop constraint UN_IDTYPE;
alter table DSY.DEVICES add constraint UN_DEVICE_ID unique(ID);

--*Migrate db2e encryption userid and pass:
update DSY.DEVICES set DB2E_USERID=(select DB2E_USERID from DSY.CLIENTS where ID=DSY.DEVICES.CLIENT_ID), DB2E_PASSWORD=(select DB2E_PASSWORD from DSY.CLIENTS where ID=DSY.DEVICES.CLIENT_ID) where DSY.DEVICES.CLIENT_ID in (select ID from DSY.CLIENTS where not DSY.CLIENTS.DB2E_USERID is null); 
--*To prevent a second run, delete migrated userids and passwords:
update DSY.CLIENTS set DB2E_USERID=NULL, DB2E_PASSWORD=NULL where DSY.CLIENTS.ID in (select CLIENT_ID from DSY.DEVICES where not DSY.DEVICES.DB2E_USERID is NULL);

--*Drop is needed for migration from a v8.1.4 bad trigger
drop trigger DSY.DEVICESTRG;
create trigger DSY.DEVICESTRG 
after delete on DSY.DEVICES 
referencing old as OLD 
for each row mode db2sql 
begin atomic 
	delete from DSY.SESSIONMONITOR where DEVICE_ID = (OLD.TYPE || '_' || OLD.ID)
; end;

--*drop old device config table:
--*drop table DSY.DEVICECONFIG;
drop table DSY.DEVICECONFIGHISTORY;

--* Create table DSY.CONFIG_CHANGES (if does not exist)
create table
  DSY.CONFIG_CHANGES(
    CONFIG_LEVEL integer primary key generated always as identity (start with 1, increment by 1, no cache),
    STATE char(1) not null,
    OBJECT_ID char(13) for bit data not null,
    OBJECT_NAME varchar(257) not null,
    OBJECT_TYPE char(1) not null,
    PARENT_OBJECT_ID char(13) for bit data not null,
    PARENT_OBJECT_NAME varchar(257) not null,
    PARENT_OBJECT_TYPE char(1) not null,
    INSERT_TIMESTAMP timestamp not null with default current timestamp,

    constraint CK_STATE check(STATE in('A','U','D'))
  );
create index DSY.CC_TIMESTAMP on DSY.CONFIG_CHANGES(INSERT_TIMESTAMP);
create index DSY.CC_OBJ_ID on DSY.CONFIG_CHANGES(OBJECT_ID);
create index DSY.CC_P_OBJ_ID on DSY.CONFIG_CHANGES(PARENT_OBJECT_ID);
create index DSY.CC_OBJ_PAR_ID on DSY.CONFIG_CHANGES(OBJECT_ID, PARENT_OBJECT_ID);


--*Update device config_level for existing devices:
update dsy.devices set config_level=0 where not exists (select max(dsy.config_changes.config_level) from dsy.config_changes) and dsy.devices.config_level=-1;
update dsy.devices set config_level=0 where (select max(dsy.config_changes.config_level) from dsy.config_changes) is NULL and dsy.devices.config_level=-1;
update dsy.devices set config_level=(select max(dsy.config_changes.config_level) from dsy.config_changes) where exists (select max(dsy.config_changes.config_level) from dsy.config_changes) and dsy.devices.config_level=-1;



--* Create table DSY.APPLICATIONS (if does not exist)
create table
  DSY.APPLICATIONS(
    ID char(13) for bit data primary key not null,
    NAME varchar(18) unique not null,
    DESCRIPTION varchar(128),
    SIGNATURE varchar(18),
    DATA_LEVEL int not null with default 0
  );
  
alter table DSY.APPLICATIONS add column DATA_LEVEL int not null with default 0;

--* Create table DSY.GROUP_APPLICATIONS (if does not exist)
--* no primary key as id's can exist many times
create table
  DSY.GROUP_APPLICATIONS(
    GROUP_ID char(13) for bit data not null,
    APPLICATION_ID  char(13) for bit data not null,
    ORDER integer with default 0,
    constraint UN_GROUP_APPL unique(GROUP_ID,APPLICATION_ID),
    constraint FK_GROUP_ID foreign key(GROUP_ID) references DSY.GROUPS(ID) on delete cascade,
    constraint FK_APPLICATION_ID foreign key(APPLICATION_ID) references DSY.APPLICATIONS(ID) on delete cascade
  );
alter table DSY.GROUP_APPLICATIONS add column ORDER integer with default 0;
create index DSY.GA_GROUP_ID on DSY.GROUP_APPLICATIONS(GROUP_ID);
create index DSY.GA_APPLICATION_ID on DSY.GROUP_APPLICATIONS(APPLICATION_ID);

update dsy.group_applications set ORDER=0 where ORDER is NULL;

--* Create table DSY.ADAPTERS (if does not exist)
create table
  DSY.ADAPTERS(
    ID char(13) for bit data primary key not null,
    NAME varchar(18) unique not null,
    DESCRIPTION varchar(128),
    FILE_ATTRIBUTES varchar(512),
    COMM_ATTRIBUTES varchar(512),
    SIGNATURE varchar(18),
    CLASSNAME varchar(512)
  );
alter table DSY.ADAPTERS add column CLASSNAME varchar(512);

--*   Insert default adapters (if do not exist)
insert into dsy.adapters(id, name, description, signature) values(x'00000000000000000000000000', 'DSYFILE',  'Default DB2 Everyplace File adapter',  'DSYFILE');
insert into dsy.adapters(id, name, description, signature) values(x'00000000000000000000000001', 'DSYDPROPR', 'Default DB2 Everyplace DataPropagator adapter', 'DSYDPROPR');
insert into dsy.adapters(id, name, description, signature) values(x'00000000000000000000000002', 'DSYJDBC', 'Default DB2 Everyplace JDBC adapter', 'DSYJDBC');
insert into dsy.adapters(id, name, description, file_attributes, comm_attributes, signature, classname) values(GENERATE_UNIQUE(), 'Agent Adapter', 'Default DB2 Everyplace Agent Adapter', 'libname=agent.dll', 'cmd=/servlet/com.ibm.mobileservices.adapter.agent;dest=127.0.0.1;8080;', 'DSYAGENT', 'AgentAdapter.class');
update dsy.adapters set name='DSYDPROPR' where name='DSYTABLE';
update dsy.adapters set signature='DSYDPROPR' where name='DSYDPROPR';
update dsy.adapters set description='Default DB2 Everyplace DataPropagator adapter' where name='DSYDPROPR';
delete from dsy.adapters where (signature='DSYPIM');

--* Create table DSY.SUBSCRIPTIONS (if does not exist)
create table
  DSY.SUBSCRIPTIONS(
    ID char(13) for bit data primary key not null,
    NAME varchar(18) unique not null,
    DESCRIPTION varchar(128),
    ADAPTER_ID char(13) for bit data,
    TIMESTAMP timestamp with default current timestamp,
    ENCRYPTION_LEVEL integer with default 0,
    BLOCK char(1) not null with default 'N',
    DEVICE_TYPES varchar(128),
    DATA_LEVEL int not null with default 0,
    BLOCK_UNTIL_REPL char(1) not null with default 'N',
    constraint CK_BLOCK check(BLOCK in('N','Y')),
    constraint CK_BLK_TIL_REPL check(BLOCK_UNTIL_REPL in('N','Y')),
    constraint FK_ADAPTER_ID foreign key(ADAPTER_ID) references DSY.ADAPTERS(ID) on delete set null
  );
alter table DSY.SUBSCRIPTIONS add column ENCRYPTION_LEVEL integer with default 0;
alter table DSY.SUBSCRIPTIONS add column BLOCK char(1) not null with default 'N' add constraint CK_BLOCK check(BLOCK in('N','Y'));
alter table DSY.SUBSCRIPTIONS add column BLOCK_UNTIL_REPL char(1) not null with default 'N' add constraint CK_BLK_TIL_REPL check(BLOCK_UNTIL_REPL in('N','Y'));
alter table DSY.SUBSCRIPTIONS add column DEVICE_TYPES varchar(128);
alter table DSY.SUBSCRIPTIONS add column DATA_LEVEL int not null with default 0;

--* Create table DSY.APPLICATION_SUBS (if does not exist)
create table
  DSY.APPLICATION_SUBS(
    APPLICATION_ID char(13) for bit data not null,
    SUBSCRIPTION_ID char(13) for bit data not null,
    ORDER integer with default 0,
    constraint UN_APPL_SUB unique(APPLICATION_ID,SUBSCRIPTION_ID),
    constraint FK_APPLICATION_ID foreign key(APPLICATION_ID) references DSY.APPLICATIONS(ID) on delete cascade,
    constraint FK_SUBSCRIPTION_ID foreign key(SUBSCRIPTION_ID) references DSY.SUBSCRIPTIONS(ID) on delete cascade
  );
alter table DSY.APPLICATION_SUBS add column ORDER integer with default 0;
create index DSY.SA_SUBSCRIPTION_ID on DSY.APPLICATION_SUBS(SUBSCRIPTION_ID);
create index DSY.SA_APPLICATION_ID on DSY.APPLICATION_SUBS(APPLICATION_ID);
update dsy.application_subs set ORDER=0 where ORDER is NULL;

--* Create table DSY.FILE_SUBSCRIPTIONS (if does not exist)
create table
  DSY.FILE_SUBSCRIPTIONS(
    SUBSCRIPTION_ID char(13) for bit data primary key not null,
    SOURCE varchar(255) not null,
    TARGET varchar(255),
    DIRTY char(1) not null with default 'Y',
    REFRESH_TIMESTAMP timestamp,
    constraint CK_DIRTY check(DIRTY in('N','Y')),
    constraint FK_SUBSCRIPTION_ID foreign key(SUBSCRIPTION_ID) references DSY.SUBSCRIPTIONS(ID) on delete cascade
  );

--* Create table DSY.REPL_MASTER (if does not exist)
create table
  DSY.REPL_MASTER(
    ID char(13) for bit data primary key not null,
    SYSTEM varchar(128) not null,
    INSTANCE varchar(8) not null,
    DATABASE varchar(18) not null,
    USERID varchar(128) not null,
    PASSWORD varchar(128) not null,
    DATABASEALIAS varchar(18),
    constraint UN_SID unique(SYSTEM,INSTANCE,DATABASE)
  );
alter table DSY.REPL_MASTER add column DATABASEALIAS varchar(18);

--* Create table DSY.REPL_MIRROR (if does not exist)
create table
  DSY.REPL_MIRROR(
    ID char(13) for bit data primary key not null,
    DATABASE varchar(18) not null,
    USERID varchar(128) not null,
    PASSWORD varchar(128) not null,
    SYNC_WINDOW integer with default 3600,
    SYNC_MAX_CLIENTS integer with default 10,
    constraint UN_DATABASE unique(DATABASE)
  );

--* Create table DSY.REPL_SUBSCRIPTIONS (if does not exist)
create table
  DSY.REPL_SUBSCRIPTIONS(
    SUBSCRIPTION_ID char(13) for bit data primary key not null,
    REPL_MASTER_ID char(13) for bit data not null,
    REPL_MIRROR_ID char(13) for bit data not null,
    TARGET_DATABASE varchar(18),
    APPLY_QUALIFIER char(18) not null,
    MIRROR_ONLY char(1) not null with default 'N',
    constraint FK_SUBSCRIPTION_ID foreign key(SUBSCRIPTION_ID) references DSY.SUBSCRIPTIONS(ID) on delete cascade,
    constraint FK_REPL_MSTR_ID foreign key(REPL_MASTER_ID) references DSY.REPL_MASTER(ID) on delete cascade,
    constraint FK_REPL_MIRR_ID foreign key(REPL_MIRROR_ID) references DSY.REPL_MIRROR(ID) on delete cascade,
    constraint CK_MIRRORONLY check(MIRROR_ONLY in('N','Y'))
  );

--* Create table DSY.REPL_SUB_TABLES (if does not exist)
create table
  DSY.REPL_SUB_TABLES(
    SUBSCRIPTION_ID char(13) for bit data not null,
    SOURCE_SCHEMA varchar(128) not null,
    SOURCE_TABLE varchar(128) not null,
    TARGET_SCHEMA varchar(128) not null,
    TARGET_TABLE varchar(128) not null,
    DELETEAUTH char(1) not null with default 'Y',
    INSERTAUTH char(1) not null with default 'Y',
    UPDATEAUTH char(1) not null with default 'Y',
    WHERE_CLAUSE varchar(1024),
    SELECTAUTH char(1) not null with default 'Y',
    ID char(13) for bit data,
    WHERE_CLAUSE2 varchar(1024),
    ORDER integer with default 0,
    ENCRYPT char(1) not null with default 'N',
		 TABLESPACE varchar(32),
		 SOURCE_TABLESPACE varchar(32),
    JOINFILTER integer with default 2,
    DATA_LEVEL int not null with default 0,    
    constraint FK_SUBSCRIPTION_ID foreign key(SUBSCRIPTION_ID) references DSY.SUBSCRIPTIONS(ID) on delete cascade,
    constraint UN_SUB_TBNAME unique(SUBSCRIPTION_ID,TARGET_SCHEMA,TARGET_TABLE),
    constraint CK_DELETEAUTH check(DELETEAUTH in('N','Y')),
    constraint CK_INSERTAUTH check(INSERTAUTH in('N','Y')),
    constraint CK_UPDATEAUTH check(UPDATEAUTH in('N','Y')),
    constraint CK_SELECTAUTH check(SELECTAUTH in('N','Y')),
    constraint CK_ENCRYPT check(ENCRYPT in('N','Y')),
    constraint CK_JOINFILTER check(JOINFILTER in(0, 1, 2))
  );
alter table DSY.REPL_SUB_TABLES add column SELECTAUTH char(1) not null with default 'Y' add constraint CK_SELECTAUTH check(SELECTAUTH in('N','Y'));
alter table DSY.REPL_SUB_TABLES add column ID char(13) for bit data;
update dsy.repl_sub_tables set id=generate_unique() where (id is null);
alter table DSY.REPL_SUB_TABLES add column WHERE_CLAUSE2 varchar(1024);
alter table DSY.REPL_SUB_TABLES add column ORDER integer with default 0;
alter table DSY.REPL_SUB_TABLES add column ENCRYPT char(1) not null with default 'N' add constraint CK_ENCRYPT check(ENCRYPT in('N','Y'));
alter table DSY.REPL_SUB_TABLES add column TABLESPACE varchar(32);
alter table DSY.REPL_SUB_TABLES add column SOURCE_TABLESPACE varchar(32);
alter table DSY.REPL_SUB_TABLES add column JOINFILTER integer with default 2 add constraint CK_JOINFILTER check(JOINFILTER in(0, 1, 2));
alter table DSY.REPL_SUB_TABLES add column DATA_LEVEL int not null with default 0;
update dsy.repl_sub_tables set selectauth='Y' where (selectauth='N' and (updateauth='Y' or deleteauth='Y'));

--*make sure they don't have nulls:
update dsy.repl_sub_tables set JOINFILTER=2 where JOINFILTER is NULL;
update dsy.repl_sub_tables set ORDER=0 where ORDER is NULL;

create index DSY.RST_SUB_ID on DSY.REPL_SUB_TABLES(SUBSCRIPTION_ID);
create index DSY.RST_ID on DSY.REPL_SUB_TABLES(ID);

--* Create table DSY.JDBC_MASTER (if does not exist)
create table
  DSY.JDBC_MASTER(
    ID char(13) for bit data primary key not null,
    DATABASE varchar(255) not null,
    DRIVER varchar(255) not null,
    USERID varchar(128) not null,
    PASSWORD varchar(128) not null,
    CONTROL_TABLESPACE varchar(51),
    constraint UN_MASTER unique(DATABASE,DRIVER)
  );
alter table DSY.JDBC_MASTER add column CONTROL_TABLESPACE varchar(51);

--* Create table DSY.JDBC_MIRROR (if does not exist)
create table
  DSY.JDBC_MIRROR(
    ID char(13) for bit data primary key not null,
    DATABASE varchar(255) not null,
    DRIVER varchar(255) not null,
    USERID varchar(128) not null,
    PASSWORD varchar(128) not null,
    SYNC_WINDOW integer with default 3600,
    SYNC_MAX_CLIENTS integer with default 10,
    constraint UN_MIRROR unique(DATABASE,DRIVER)
  );
  
update dsy.jdbc_mirror set SYNC_WINDOW=0 where SYNC_WINDOW is NULL;
update dsy.jdbc_mirror set SYNC_MAX_CLIENTS=0 where SYNC_MAX_CLIENTS is NULL;
  
--* Create table DSY.JDBC_SUBSCRIPTIONS (if does not exist)
create table
  DSY.JDBC_SUBSCRIPTIONS(
    SUBSCRIPTION_ID char(13) for bit data primary key not null,
    JDBC_MASTER_ID char(13) for bit data not null,
    JDBC_MIRROR_ID char(13) for bit data,
    TARGET_DATABASE varchar(18),
    constraint FK_SUBSCRIPTION_ID foreign key(SUBSCRIPTION_ID) references DSY.SUBSCRIPTIONS(ID) on delete cascade,
    constraint FK_JDBC_MSTR_ID foreign key(JDBC_MASTER_ID) references DSY.JDBC_MASTER(ID) on delete cascade
  );
alter table DSY.JDBC_SUBSCRIPTIONS drop constraint FK_JDBC_MIRR_ID;

--* Create table DSY.RDB_SUB_COLUMNS (if does not exist)
create table
  DSY.RDB_SUB_COLUMNS(
    ID char(13) for bit data primary key not null,
    REPL_SUB_TABLE_ID char(13) for bit data not null,
    SOURCE varchar(512) not null,
    TARGET varchar(128) not null,
    SUBSCRIBE char(1) not null with default 'N',
    REPLICATE char(1) not null with default 'N',
    KEY char(1) not null with default 'N',
    CONSTRNT varchar(1024),
    DATAFILTER varchar(18),
--*    AUTOKEY char(1) not null with default 'N',
    constraint UN_RPLIDTRG unique(REPL_SUB_TABLE_ID,TARGET),
    constraint CK_SUBSCRIBE check(SUBSCRIBE in('N','Y')),
    constraint CK_REPLICATE check(REPLICATE in('N','Y')),
    constraint CK_KEY check(KEY in('N','Y'))
--*    constraint CK_AUTOKEY check(AUTOKEY in('N','Y'))
--* not unique/primary key constraint FK_REPL_SUB_ID foreign key(REPL_SUB_TABLE_ID) references DSY.REPL_SUB_TABLES(ID) on delete cascade
  );
--* alter table DSY.REPL_SUB_COLUMNS add column AUTOKEY char(1) not null with default 'N' add constraint CK_AUTOKEY check(AUTOKEY in('N','Y'));
drop table DSY.JDBC_SUB_COLUMNS;

--* Create table DSY.RDB_SUB_INDEXES (if does not exist)
create table
  DSY.RDB_SUB_INDEXES(
    REPL_SUB_TABLE_ID char(13) for bit data not null,
    ID char(13) for bit data primary key not null,
    NAME varchar(18) not null unique,
    COLNAMES varchar(1024) not null
  );
create index DSY.RSI_SUB_TABLE_ID on DSY.RDB_SUB_INDEXES(REPL_SUB_TABLE_ID);

--* Create table DSY.CUSTOM_SUBSCRIPTIONS (if does not exist)
create table
  DSY.CUSTOM_SUBSCRIPTIONS(
    SUBSCRIPTION_ID char(13) for bit data primary key not null,
    OTHER clob(4096),
    constraint FK_SUBSCRIPTION_ID foreign key(SUBSCRIPTION_ID) references DSY.SUBSCRIPTIONS(ID) on delete cascade
  );

--* Create table DSY.GROUP_PROPERTIES (if does not exist)
create table
  DSY.GROUP_PROPERTIES(
    GROUP_ID char(13) for bit data not null,
    NAME varchar(18) not null,
    VALUE varchar(36),
    constraint UN_GROUP_NAME unique(GROUP_ID,NAME),
    constraint FK_GROUP_ID foreign key(GROUP_ID) references DSY.GROUPS(ID) on delete cascade
  );

--* Create table DSY.CLIENT_PROPERTIES (if does not exist)
create table
  DSY.CLIENT_PROPERTIES(
    CLIENT_ID char(13) for bit data not null,
    NAME varchar(18) not null,
    VALUE varchar(36),
    constraint UN_CLIENT_NAME unique(CLIENT_ID,NAME),
    constraint FK_CLIENT_ID foreign key(CLIENT_ID) references DSY.CLIENTS(ID) on delete cascade
  );

--* Create table DSY.LOG (if does not exist)
create table
  DSY.LOG(
    ID char(13) for bit data primary key not null,
    TIMESTAMP timestamp not null with default current timestamp,
    MSGNUM varchar(8) not null,
    MSGTEXT varchar(512) not null,
    USERNAME varchar(254),
    SUBSCRIPTIONNAME varchar(18),
    DATABASENAME varchar(255),
    HOSTPORT varchar(270)
  );
alter table DSY.LOG add column USERNAME varchar(254);
alter table DSY.LOG add column SUBSCRIPTIONNAME varchar(18);
alter table DSY.LOG add column DATABASENAME varchar(255);
alter table DSY.LOG add column HOSTPORT varchar(270);
alter table DSY.LOG alter column USERNAME set data type varchar(254);
alter table DSY.LOG alter column DATABASENAME set data type varchar(255);

--* Drop table DSY.TABLEMAPPER (if it exists)
drop table dsy.tablemapper;

--* Create table DSY.SERVERS (if does not exist)
create table
  DSY.SERVERS(
    NAME varchar(128) primary key not null,
    DESCRIPTION VARCHAR(128),
    HOST varchar(128) not null,
    PORT INTEGER not null with default 8080,
    SYNCENABLED CHAR(1) not null with default 'N',
    REPLENABLED CHAR(1) not null with default 'N',
    BLOCKED CHAR(1) not null with default 'N',
    STATUS INTEGER not null with default 0,
    constraint UN_HOSTPORT unique(HOST,PORT),
    constraint CK_SYNCENABLED check(SYNCENABLED in('N','Y')),
    constraint CK_REPLENABLED check(REPLENABLED in('N','Y')),
    constraint CK_BLOCKED check(BLOCKED in('N','Y'))
  );

--* Create table DSY.CL_IMPL_MAP (if does not exist)
create table
  DSY.CL_IMPL_MAP(
    OBJECT_ID char(13) for bit data not null,
    TYPE int not null with default 0,
    CLASSNAME varchar(255),
    ENABLED char(1) not null with default 'Y',
    constraint UN_IDTYPE unique(OBJECT_ID,TYPE),
    constraint CK_ENABLED check(ENABLED in('N','Y'))
  );

--* Create table DSY.PROPERTIES (if does not exist)
create table
  DSY.PROPERTIES(
    TYPE varchar(255) NOT NULL,
    NAME varchar(255) NOT NULL,
    VALUE varchar(1024),
    DESCRIPTION varchar(1024),
    constraint UN_TYPENAME unique(TYPE,NAME)
  );
--* Refer to dsyproperties.ddl for properties that use to be in this .ddl file



--* TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS
--* TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS
--* TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS
--* TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS
--* TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS
--* TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS TRIGGERS

--* Create trigger to reset devices of a client if client is assigned to a
--* new group. Also to delete client properties that are not present in it's group.
drop trigger DSY.UPD_CLIENT_GRP;
create trigger DSY.UPD_CLIENT_GRP
after update of GROUP_ID on DSY.CLIENTS
referencing NEW as N OLD as O 
for each row mode db2sql
when(N.group_id is NULL or not N.group_id=O.group_id)
begin atomic
	delete from dsy.devices where client_id=N.id;--*
	delete from dsy.client_properties where client_id=N.id and name not in (select gp.name from dsy.group_properties gp where gp.group_id=N.group_id)
; end;


--*create trigger to reject inserts if there would be duplicate 
drop trigger DSY.UNQ_GRP_TBLS_1; 
CREATE TRIGGER DSY.UNQ_GRP_TBLS_1 
NO CASCADE BEFORE  INSERT  ON DSY.GROUP_APPLICATIONS  
REFERENCING  NEW AS N  
FOR EACH ROW  MODE DB2SQL 
when(exists(
(select t.target_schema, t.target_table
from 
dsy.repl_sub_tables t, 
dsy.application_subs sa, 
dsy.group_applications ga
where 
t.subscription_id = sa.subscription_id and 
sa.application_id = ga.application_id and ga.group_id=N.group_id) 
INTERSECT
(select t.target_schema, t.target_table
from 
dsy.repl_sub_tables t, 
dsy.application_subs sa
where 
t.subscription_id = sa.subscription_id and 
sa.application_id=N.application_id)
))
BEGIN ATOMIC
     signal sqlstate '99008' ('Duplicate target tables in group'); END;


--*  A group should not have duplicate target_schema and target_table.
--*  The first select finds all the groups that contain the given subscriptions tables,
--*  then it intersects this result with the groups of the subscription that we
--*  are trying to add into this subscription set. 
--*  If intersection is not empty set, then this a duplicate subscription in this group.;
drop trigger DSY.UNQ_GRP_TBLS_3;
create trigger DSY.UNQ_GRP_TBLS_3 
no cascade before insert on DSY.APPLICATION_SUBS  
referencing NEW as N 
for each row mode db2sql 
when(exists(select ga.group_id, t.target_schema, t.target_table from
	dsy.repl_sub_tables t, 
	dsy.application_subs sa,
	dsy.group_applications ga  
	where t.subscription_id=sa.subscription_id and ga.application_id=sa.application_id 
	INTERSECT 
	select ga.group_id, t.target_schema, t.target_table from
	dsy.group_applications ga,
	dsy.repl_sub_tables t    
	where
	t.subscription_id=N.subscription_id and ga.application_id=N.application_id) 
)	
BEGIN ATOMIC
     signal sqlstate '99008' ('Duplicate target tables in group'); END;


--*Create trigger to detect and reject duplicate target tables in the same subscription set.
drop trigger DSY.UNQ_APP_TBLS_1; 
CREATE TRIGGER DSY.UNQ_APP_TBLS_1  
NO CASCADE BEFORE  INSERT  ON DSY.APPLICATION_SUBS 
REFERENCING  NEW AS N  
FOR EACH ROW  MODE DB2SQL 
when(exists(
(select t.target_schema, t.target_table 
from 
dsy.repl_sub_tables t, 
dsy.application_subs sa 
where 
t.subscription_id = sa.subscription_id and 
sa.application_id = N.application_id) 
INTERSECT
(select t.target_schema, t.target_table 
from 
dsy.repl_sub_tables t  
where 
t.subscription_id = N.subscription_id)
))
BEGIN ATOMIC
     signal sqlstate '99009' ('Duplicate target tables in subscription set'); END;


--* Create trigger DSY.UNQ_INS_REPL_MIRR to make sure a new repl mirror
--* is not already in use in DSY.JDBC_MIRROR table.
drop trigger DSY.UNQ_INS_REPL_MIRR;
create trigger DSY.UNQ_INS_REPL_MIRR
no cascade before insert on DSY.REPL_MIRROR
referencing  NEW as N  
for each row mode db2sql     
when (('JDBC:DB2:' || UCASE(N.database)) in (Select UCASE(database) from DSY.JDBC_MIRROR))
begin atomic
  signal sqlstate '99001' ('Database is already listed as JDBC mirror -- it can not be used as DPropR mirror'); end;

--* Create trigger DSY.UNQ_UPD_REPL_MIRR to make sure an updated repl mirror
--* is not already in use in DSY.JDBC_MIRROR table.
drop trigger DSY.UNQ_UPD_REPL_MIRR;
create trigger DSY.UNQ_UPD_REPL_MIRR
no cascade before update of database on DSY.REPL_MIRROR
referencing  NEW as N  
for each row mode db2sql     
when (('JDBC:DB2:' || UCASE(N.database)) in (Select UCASE(database) from DSY.JDBC_MIRROR))
begin atomic
  signal sqlstate '99001' ('Database is already listed as JDBC mirror --  it can not be used as DPropR mirror'); end;  

--* Create trigger DSY.REPL_MIRR_INUSE to make sure a new repl mirror
--* is not in use by a subscription.
drop trigger DSY.REPL_MIRR_INUSE;
create trigger DSY.REPL_MIRR_INUSE
no cascade before delete on DSY.REPL_MIRROR
referencing  OLD as O  
for each row mode db2sql     
when ( exists(select subscription_id from DSY.REPL_SUBSCRIPTIONS rs where rs.repl_mirror_id=O.id) )
begin atomic
  signal sqlstate '99003' ('Mirror database in use by a subscription'); end;

--* Create trigger DSY.REPL_MAST_INUSE to make sure a new repl mirror
--* is not in use by a subscription.
drop trigger DSY.REPL_MAST_INUSE;
create trigger DSY.REPL_MAST_INUSE
no cascade before delete on DSY.REPL_MASTER
referencing  OLD as O  
for each row mode db2sql     
when ( exists(select subscription_id from DSY.REPL_SUBSCRIPTIONS rs where rs.repl_master_id=O.id) )
begin atomic
  signal sqlstate '99004' ('Master database in use by a subscription'); end;


--* Create trigger DSY.UNQ_INS_JDBC_MIRR to make sure a new jdbc mirror
--* is not already in use in DSY.REPL_MIRROR table.
drop trigger DSY.UNQ_INS_JDBC_MIRR;
create trigger DSY.UNQ_INS_JDBC_MIRR
no cascade before insert on DSY.JDBC_MIRROR
referencing  NEW as N  
for each row mode db2sql     
when (UCASE(N.database) in (Select 'JDBC:DB2:' || UCASE(database) from DSY.REPL_MIRROR))
begin atomic
  signal sqlstate '99002' ('Database is already listed as DPropR mirror -- it can not be used as JDBC mirror'); end;  

--* Create trigger DSY.UNQ_UPD_JDBC_MIRR to make sure an updated jdbc mirror
--* is not already in use in DSY.REPL_MIRROR table.
drop trigger DSY.UNQ_UPD_JDBC_MIRR;
create trigger DSY.UNQ_UPD_JDBC_MIRR
no cascade before update of database on DSY.JDBC_MIRROR
referencing  NEW as N  
for each row mode db2sql     
when (UCASE(N.database) in (Select 'JDBC:DB2:' || UCASE(database) from DSY.REPL_MIRROR))
begin atomic
  signal sqlstate '99002' ('Database is already listed as DPropR mirror -- it can not be used as JDBC mirror'); end;  

--* Create trigger DSY.JDBC_MIRR_INUSE to make sure a new repl mirror
--* is not in use by a subscription.
drop trigger DSY.JDBC_MIRR_INUSE;
create trigger DSY.JDBC_MIRR_INUSE
no cascade before delete on DSY.JDBC_MIRROR
referencing  OLD as O  
for each row mode db2sql     
when ( exists(select subscription_id from DSY.JDBC_SUBSCRIPTIONS js where js.jdbc_mirror_id=O.id) )
begin atomic
  signal sqlstate '99003' ('Mirror database in use by a subscription'); end;

--* Create trigger DSY.JDBC_MAST_INUSE to make sure a new repl mirror
--* is not in use by a subscription.
drop trigger DSY.JDBC_MAST_INUSE;
create trigger DSY.JDBC_MAST_INUSE
no cascade before delete on DSY.JDBC_MASTER
referencing  OLD as O  
for each row mode db2sql     
when ( exists(select subscription_id from DSY.JDBC_SUBSCRIPTIONS js where js.jdbc_master_id=O.id) )
begin atomic
  signal sqlstate '99004' ('Master database in use by a subscription'); end;


--* Create trigger DSY.JDBC_MAST_INUSE2 to make sure a new repl mirror
--* is not in use by a subscription.
drop trigger DSY.JDBC_MAST_INUSE2;
create trigger DSY.JDBC_MAST_INUSE2
no cascade before update of USERID on DSY.JDBC_MASTER
referencing  NEW as N OLD as O
for each row mode db2sql     
when ((not O.USERID=N.USERID) and exists(select subscription_id from DSY.JDBC_SUBSCRIPTIONS js where js.jdbc_master_id=O.id) )
begin atomic
  signal sqlstate '99011' ('Master database in use by a subscription -- cannot change USERID once it is used as it serves as the schema name for control tables in master database'); end;



--* Create trigger DSY.ADAPTER_INUSE to make sure an adapter being deleted
--* is not in use by a subscription.
drop trigger DSY.ADAPTER_INUSE;
create trigger DSY.ADAPTER_INUSE
no cascade before delete on DSY.ADAPTERS
referencing  OLD as O  
for each row mode db2sql     
when ( exists(select adapter_id from DSY.SUBSCRIPTIONS s where s.adapter_id=O.id) )
begin atomic
  signal sqlstate '99010' ('Adapter in use by a subscription'); end;




--*create trigger to auto delete columns and indexes of a deleted table.
--*drop trigger DSY.DEL_RDB_SUB_TABLE;
--*create trigger DSY.DEL_RDB_SUB_TABLE 
--*after delete on DSY.REPL_SUB_TABLES  
--*referencing OLD AS O  
--*for each row mode db2sql 
--*begin atomic
--*DSY.RST_SUBSCRIPTION_ID     delete from dsy.rdb_sub_columns c where c.repl_sub_table_id=O.id; delete from dsy.rdb_sub_indexes i where i.repl_sub_table_id=O.id; end;


--*SUBSCRIPTION_SET Related Device Config Triggers:

drop trigger DSY.INS_CFG_GRP_APP;
create trigger DSY.INS_CFG_GRP_APP
after insert on DSY.GROUP_APPLICATIONS
referencing NEW as N  
for each row mode db2sql     
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)
select 'A', 
	a.id, a.name, 'A', 
	g.id, g.name, 'G'   
from dsy.applications a, dsy.groups g 
where a.id=N.application_id and g.id=N.group_id
; end;

--*This trigger may get fired after a group is deleted, or an app is deleted as well as a plain row delete from it.
--*We do not need to log anything in case the group is deleted.
drop trigger DSY.DEL_CFG_GRP_APP;
create trigger DSY.DEL_CFG_GRP_APP
after delete on DSY.GROUP_APPLICATIONS
referencing OLD as O  
for each row mode db2sql     
begin atomic

--*Note that dsy.group_applications(which triggers this) is not used in joins.
insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type) 
select 'D', 
	O.application_id, 
	COALESCE((select a.name from dsy.applications a where a.id=O.application_id),O.application_id), 'A', 
	g.id, g.name, 'G'  
from dsy.groups g   
where g.id=O.group_id 
; end;

--*track changes to the order of applications, assuming ORDER is not null.
drop trigger DSY.UPD_CFG_GRP_APP;
create trigger DSY.UPD_CFG_GRP_APP
after update of ORDER on DSY.GROUP_APPLICATIONS
referencing NEW as N OLD as O 
for each row mode db2sql  
when(not N.order=O.order)
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', 
	a.id, a.name, 'A', 
	g.id, g.name, 'G'  
from dsy.applications a, dsy.groups g 
where a.id=N.application_id and g.id=N.group_id
; end;


drop trigger DSY.UPD_CFG_APP;
create trigger DSY.UPD_CFG_APP
after update of NAME on DSY.APPLICATIONS
referencing NEW as N OLD as O 
for each row mode db2sql  
when(not (N.name=O.name))   
begin atomic
insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('U', O.id, O.name, 'A', x'00000000000000000000000000', '', 'G');--*

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', 
	a.id, a.name, 'A', 
	g.id, g.name, 'G'  
from dsy.applications a, dsy.group_applications ga, dsy.groups g 
where a.id=N.id and ga.application_id=a.id and g.id=ga.group_id
; end;


--*Create trigger that populates DSY.CONFIG_CHANGES after 
--*a delete from DSY.APPLICATIONS.
--*Note that at this point we cannot access the groups
--*effected by this change because dsy. have already cascade deleted.
--*Therefore, we have another after delete trigger on dsy.group_applications that records
--*the relevant entries by inserting application id for application name field. 
--*That trigger cannot access the dsy.applications.name field since the trigger on
--*dsy.group_applications is fired after the row is deleted from dsy.applications table. 
--*So in this trigger, we will find the rows which were inserted by the other trigger and 
--*update them to reflect correct name since we now have access to OLD.name of the application.

drop trigger DSY.DEL_CFG_APP;
create trigger DSY.DEL_CFG_APP
after delete on DSY.APPLICATIONS
referencing OLD as O 
for each row mode db2sql     
begin atomic
insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('D', O.id, O.name, 'A', x'00000000000000000000000000', '', 'G'); --*

update dsy.config_changes 
set object_name=O.name where
object_id=O.id and object_name=object_id
; end;


--*SUBSCRIPTION Related Device Config Triggers:

drop trigger DSY.INS_CFG_APP_SUBS;
create trigger DSY.INS_CFG_APP_SUBS
after insert on DSY.APPLICATION_SUBS
referencing NEW as N  
for each row mode db2sql     
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'A', 
	s.id, s.name, 'S', 
	a.id, a.name, 'A'  
from dsy.applications a, dsy.subscriptions s  
where a.id=N.application_id and s.id=N.subscription_id
; end;
	

drop trigger DSY.DEL_CFG_APP_SUBS;
create trigger DSY.DEL_CFG_APP_SUBS
after delete on DSY.APPLICATION_SUBS
referencing OLD as O   
for each row mode db2sql     
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type) 
select 'D', 
	O.subscription_id, 
	COALESCE((select s.name from dsy.subscriptions s where s.id=O.subscription_id),O.subscription_id), 'S', 
	a.id, a.name, 'A'  
from dsy.applications a  
where a.id=O.application_id 
; end;

--*track changes to the order of subscriptions assuming ORDER is not null.
drop trigger DSY.UPD_CFG_APP_SUBS;
create trigger DSY.UPD_CFG_APP_SUBS
after update of ORDER on DSY.APPLICATION_SUBS
referencing NEW as N OLD as O  
for each row mode db2sql   
when(not N.order=O.order)  
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', 
	s.id, s.name, 'S', 
	a.id, a.name, 'A'  
from dsy.applications a, dsy.subscriptions s  
where a.id=N.application_id and s.id=N.subscription_id
; end;


drop trigger DSY.UPD_CFG_SUBS;
create trigger DSY.UPD_CFG_SUBS
after update of NAME, ENCRYPTION_LEVEL, DEVICE_TYPES on DSY.SUBSCRIPTIONS
referencing NEW as N OLD as O  
for each row mode db2sql   
when((O.device_types is NULL and N.device_types is not NULL) 
			or not (N.name=O.name and 
			N.encryption_level=O.encryption_level and
			N.device_types=O.device_types)) 
begin atomic
insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('U', O.id, O.name, 'S', x'00000000000000000000000000', '', 'A');--*

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', 
	s.id, s.name, 'S', 
	a.id, a.name, 'A'  
from dsy.applications a, dsy.application_subs sa, dsy.subscriptions s  
where a.id=sa.application_id and sa.subscription_id=s.id and s.id=N.id
; end;

drop trigger DSY.UPD_CFG_FILESUBS;
create trigger DSY.UPD_CFG_FILESUBS
after update of REFRESH_TIMESTAMP on DSY.FILE_SUBSCRIPTIONS
referencing NEW as N OLD as O  
for each row mode db2sql   
when((N.refresh_timestamp is NULL or O.refresh_timestamp is NULL)
		or not (N.refresh_timestamp=O.refresh_timestamp)) 
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', 
	s.id, s.name, 'S', 
	a.id, a.name, 'A'  
from dsy.applications a, dsy.application_subs sa, dsy.subscriptions s  
where a.id=sa.application_id and sa.subscription_id=s.id and s.id=N.subscription_id
; end;


drop trigger DSY.UPD_CFG_CUSTOMSUBS;
create trigger DSY.UPD_CFG_CUSTOMSUBS
after update of OTHER on DSY.CUSTOM_SUBSCRIPTIONS
referencing NEW as N OLD as O  
for each row mode db2sql   
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', 
	s.id, s.name, 'S', 
	a.id, a.name, 'A' 
from dsy.applications a, dsy.application_subs sa, dsy.subscriptions s  
where a.id=sa.application_id and sa.subscription_id=s.id and s.id=N.subscription_id
; end;


drop trigger DSY.DEL_CFG_SUB;
create trigger DSY.DEL_CFG_SUB
after delete on DSY.SUBSCRIPTIONS
referencing OLD as O 
for each row mode db2sql     
begin atomic
insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('D', O.id, O.name, 'S', x'00000000000000000000000000', '', 'A');--*

update dsy.config_changes 
set object_name=O.name where
object_id=O.id and object_name=object_id
; end;


--*repl_sub_table related triggers:
--*a mirror database should not have duplicate target_schema and target_table.
--*The first 'exists' finds all the jdbc mirrors (jdbc_mirror) that contain the given table,
--*then it intersects this result with the mirror of the subscription that we
--*are trying to insert. If intersection is not empty set, then this a duplicate.
--*The second 'exists' does the same for dpropr mirrors (repl_mirror).
drop trigger DSY.UNQ_MIRR_TBLS;
create trigger DSY.UNQ_MIRR_TBLS 
no cascade before insert on DSY.REPL_SUB_TABLES  
referencing NEW as N 
for each row mode db2sql 
when(exists(select t.target_schema, t.target_table, jm.id from
	dsy.repl_sub_tables t, 
	dsy.jdbc_subscriptions js, 
	dsy.jdbc_mirror jm 
	where 
	t.target_schema=N.target_schema and t.target_table=N.target_table and
	t.subscription_id=js.subscription_id and js.jdbc_mirror_id=jm.id
	INTERSECT
	select N.target_schema, N.target_table, jm.id from
	dsy.jdbc_subscriptions js,
	dsy.jdbc_mirror jm
	where
	js.subscription_id=N.subscription_id and js.jdbc_mirror_id=jm.id
	) 
or 
exists(select t.target_schema, t.target_table, rm.id from
	dsy.repl_sub_tables t, 
	dsy.repl_subscriptions rs, 
	dsy.repl_mirror rm 
	where 
	t.target_schema=N.target_schema and t.target_table=N.target_table and
	t.subscription_id=rs.subscription_id and rs.repl_mirror_id=rm.id 
	INTERSECT
	select N.target_schema, N.target_table, rm.id from
	dsy.repl_subscriptions rs,
	dsy.repl_mirror rm
	where
	rs.subscription_id=N.subscription_id and rs.repl_mirror_id=rm.id
	)
)
BEGIN ATOMIC
     signal sqlstate '99007' ('Duplicate tables in mirror'); END;


--*a subscription set should not have duplicate target_schema and target_table.
--*The first 'exists' finds all the subscription sets that contain the given table,
--*then it intersects this result with the subscription sets of the subscription that we
--*are trying to insert this table into. If intersection is not empty set, then this a duplicate.
drop trigger DSY.UNQ_APP_TBLS_2;
create trigger DSY.UNQ_APP_TBLS_2 
no cascade before insert on DSY.REPL_SUB_TABLES  
referencing NEW as N 
for each row mode db2sql 
when(exists(select sa.application_id, t.target_schema, t.target_table from
	dsy.repl_sub_tables t, 
	dsy.application_subs sa 
	where 
	t.target_schema=N.target_schema and t.target_table=N.target_table and
	t.subscription_id=sa.subscription_id 
	INTERSECT
	select sa.application_id, N.target_schema, N.target_table from
	dsy.application_subs sa  
	where
	sa.subscription_id=N.subscription_id) 
)	
BEGIN ATOMIC
     signal sqlstate '99009' ('Duplicate target tables in subscription set'); END;


--*a group should not have duplicate target_schema and target_table.
--*The first 'exists' finds all the groups that contain the given table,
--*then it intersects this result with the groups of the subscription that we
--*are trying to insert this table into. If intersection is not empty set, then this a duplicate.
drop trigger DSY.UNQ_GRP_TBLS_2;
create trigger DSY.UNQ_GRP_TBLS_2 
no cascade before insert on DSY.REPL_SUB_TABLES  
referencing NEW as N 
for each row mode db2sql 
when(exists(select ga.group_id, t.target_schema, t.target_table from
	dsy.repl_sub_tables t, 
	dsy.application_subs sa,
	dsy.group_applications ga  
	where 
	t.target_schema=N.target_schema and t.target_table=N.target_table and
	t.subscription_id=sa.subscription_id and ga.application_id=sa.application_id 
	INTERSECT 
	select ga.group_id, N.target_schema, N.target_table from
	dsy.application_subs sa,
	dsy.group_applications ga   
	where
	sa.subscription_id=N.subscription_id and ga.application_id=sa.application_id) 
)	
BEGIN ATOMIC
     signal sqlstate '99008' ('Duplicate target tables in group'); END;


--*enforce unique tables for jdbc_mirror:
drop trigger DSY.UNQ_INS_JDBC_TBLS; 
create trigger DSY.UNQ_INS_JDBC_TBLS 
no cascade before insert on DSY.JDBC_SUBSCRIPTIONS   
referencing NEW as N 
for each row mode db2sql 
when(exists(select t.target_schema, t.target_table, jm.id from
	dsy.repl_sub_tables t, 
	dsy.jdbc_subscriptions js, 
	dsy.jdbc_mirror jm 
	where 
	t.subscription_id=js.subscription_id and js.jdbc_mirror_id=jm.id
	INTERSECT
	select t.target_schema, t.target_table, jm.id from
	dsy.repl_sub_tables t, 
	dsy.jdbc_mirror jm
	where
	N.subscription_id=t.subscription_id and N.jdbc_mirror_id=jm.id
	) 
)
BEGIN ATOMIC
     signal sqlstate '99007' ('Duplicate tables in mirror'); END;


--*enforce unique tables for repl_mirror:
drop trigger DSY.UNQ_INS_REPL_TBLS; 
create trigger DSY.UNQ_INS_REPL_TBLS 
no cascade before insert on DSY.REPL_SUBSCRIPTIONS   
referencing NEW as N 
for each row mode db2sql 
when(exists(select t.target_schema, t.target_table, rm.id from
	dsy.repl_sub_tables t, 
	dsy.repl_subscriptions rs, 
	dsy.repl_mirror rm 
	where 
	t.subscription_id=rs.subscription_id and rs.repl_mirror_id=rm.id
	INTERSECT
	select t.target_schema, t.target_table, rm.id from
	dsy.repl_sub_tables t, 
	dsy.repl_mirror rm
	where
	N.subscription_id=t.subscription_id and N.repl_mirror_id=rm.id
	)
)
BEGIN ATOMIC
     signal sqlstate '99007' ('Duplicate tables in mirror'); END;


--*enforce unique tables for jdbc_mirror:
drop trigger DSY.UNQ_UPD_JDBC_TBLS; 
create trigger DSY.UNQ_UPD_JDBC_TBLS 
no cascade before update of JDBC_MIRROR_ID on DSY.JDBC_SUBSCRIPTIONS 
referencing NEW as N OLD as O 
for each row mode db2sql 
when((O.jdbc_mirror_id is NULL or 
		(not O.jdbc_mirror_id=N.jdbc_mirror_id)) and 
	exists(select t.target_schema, t.target_table, jm.id from
	dsy.repl_sub_tables t, 
	dsy.jdbc_subscriptions js, 
	dsy.jdbc_mirror jm 
	where 
	t.subscription_id=js.subscription_id and js.jdbc_mirror_id=jm.id
	INTERSECT
	select t.target_schema, t.target_table, jm.id from
	dsy.repl_sub_tables t, 
	dsy.jdbc_mirror jm
	where
	N.subscription_id=t.subscription_id and N.jdbc_mirror_id=jm.id
	) 
) 
BEGIN ATOMIC 
     signal sqlstate '99007' ('Duplicate tables in mirror'); END;


--*enforce unique tables for repl_mirror:
drop trigger DSY.UNQ_UPD_REPL_TBLS; 
create trigger DSY.UNQ_UPD_REPL_TBLS 
no cascade before update of REPL_MIRROR_ID on DSY.REPL_SUBSCRIPTIONS 
referencing NEW as N OLD as O 
for each row mode db2sql 
when((O.repl_mirror_id is NULL or 
		(not O.repl_mirror_id=N.repl_mirror_id)) and 
	exists(select t.target_schema, t.target_table, rm.id from
	dsy.repl_sub_tables t, 
	dsy.repl_subscriptions rs, 
	dsy.repl_mirror rm 
	where 
	t.subscription_id=rs.subscription_id and rs.repl_mirror_id=rm.id
	INTERSECT
	select t.target_schema, t.target_table, rm.id from
	dsy.repl_sub_tables t, 
	dsy.repl_mirror rm
	where
	N.subscription_id=t.subscription_id and N.repl_mirror_id=rm.id
	)
)
BEGIN ATOMIC
     signal sqlstate '99007' ('Duplicate tables in mirror'); END;

--*mlp#9036 table insert must have selectAuth=Y if updateAuth|deleteAuth=Y
drop trigger DSY.INS_SELAUTH_TBL;
create trigger DSY.INS_SELAUTH_TBL
no cascade before insert on DSY.REPL_SUB_TABLES  
referencing NEW as N 
for each row mode db2sql 
when(n.selectAuth='N' and (updateAuth='Y' or deleteAuth='Y'))
BEGIN ATOMIC
     signal sqlstate '99012' ('Table must have selectAuth=Y as updateAuth=Y or deleteAuth=Y'); END;

--*mlp#9036 table update must have selectAuth=Y if updateAuth|deleteAuth=Y
drop trigger DSY.UPD_SELAUTH_TBL;
create trigger DSY.UPD_SELAUTH_TBL
no cascade before update of SELECTAUTH,UPDATEAUTH,DELETEAUTH on DSY.REPL_SUB_TABLES  
referencing NEW as N OLD as O 
for each row mode db2sql 
when(N.selectAuth='N' and (N.insertAuth='Y' or N.deleteAuth='Y') and not N.selectAuth=O.selectAuth)
BEGIN ATOMIC
     signal sqlstate '99012' ('Table must have selectAuth=Y as updateAuth=Y or deleteAuth=Y'); END;

drop trigger DSY.INS_CFG_REPL_TBL;
create trigger DSY.INS_CFG_REPL_TBL
after insert on DSY.REPL_SUB_TABLES
referencing NEW as N
for each row mode db2sql
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'A', 
	N.id, N.target_table, 'T', 
	s.id, s.name, 'S'  
from dsy.subscriptions s    
where s.id=N.subscription_id
; end;


drop trigger DSY.UPD_CFG_REPL_TBL;
create trigger DSY.UPD_CFG_REPL_TBL
after update of TARGET_SCHEMA, TARGET_TABLE, DELETEAUTH, 
				INSERTAUTH, UPDATEAUTH, SELECTAUTH, 
				WHERE_CLAUSE, ORDER, ENCRYPT on DSY.REPL_SUB_TABLES
referencing NEW as N OLD as O  
for each row mode db2sql   
when((N.WHERE_CLAUSE is NULL and O.WHERE_CLAUSE is not NULL) or
		(O.WHERE_CLAUSE is NULL and N.WHERE_CLAUSE is not NULL) or  
		not (N.TARGET_SCHEMA=O.TARGET_SCHEMA and
			N.TARGET_TABLE=O.TARGET_TABLE and 
			N.DELETEAUTH=O.DELETEAUTH and
			N.INSERTAUTH=O.INSERTAUTH and
			N.UPDATEAUTH=O.UPDATEAUTH and
			N.SELECTAUTH=O.SELECTAUTH and
			N.WHERE_CLAUSE=O.WHERE_CLAUSE and
			N.ORDER=O.ORDER and
			N.ENCRYPT=O.ENCRYPT)) 
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type) 
select 'U', 
	O.id, O.target_table, 'T', 
	s.id, s.name, 'S'  
from dsy.subscriptions s  
where s.id=N.subscription_id 
; end;


--*Below trigger would not work for rdb_sub_columns and
--*rdb_sub_indexes if there were foreign keys with cascade delete.
--*However, those foreign keys are not formally defined at this time.
drop trigger DSY.DEL_CFG_REPL_TBL;
create trigger DSY.DEL_CFG_REPL_TBL 
after delete on DSY.REPL_SUB_TABLES 
referencing OLD as O  
for each row mode db2sql     
begin atomic 

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'D', 
	c.id, c.target, 'C', 
	O.id, O.target_table, 'T'   
from dsy.rdb_sub_columns c  
where c.repl_sub_table_id=O.id;--*

delete from dsy.rdb_sub_columns c 
where c.repl_sub_table_id=O.id;--*

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'D', 
	i.id, i.name, 'I', 
	O.id, O.target_table, 'T'   
from dsy.rdb_sub_indexes i 
where i.repl_sub_table_id=O.id;--*

delete from dsy.rdb_sub_indexes i 
where i.repl_sub_table_id=O.id;--*

delete from dsy.cl_impl_map cl 
where cl.object_id=O.id;--*

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'D', 
	O.id, O.target_table, 'T', 
	s.id, s.name, 'S'   
from dsy.subscriptions s 
where s.id=O.subscription_id
; end;


--*rdb_sub_columns related triggers:

drop trigger DSY.INS_CFG_RDB_COL;
create trigger DSY.INS_CFG_RDB_COL
after insert on DSY.RDB_SUB_COLUMNS
referencing NEW as N  
for each row mode db2sql     
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'A', 
	N.id, N.target, 'C', 
	t.id, t.target_table, 'T' 
from dsy.repl_sub_tables t 
where t.id=N.repl_sub_table_id  
; end;


drop trigger DSY.UPD_CFG_RDB_COL;
create trigger DSY.UPD_CFG_RDB_COL
after update of TARGET, SUBSCRIBE, KEY, CONSTRNT on DSY.RDB_SUB_COLUMNS
referencing NEW as N OLD as O  
for each row mode db2sql   
when(not (N.TARGET=O.TARGET and
			N.SUBSCRIBE=O.SUBSCRIBE and 
			N.KEY=O.KEY and
			N.CONSTRNT=O.CONSTRNT)) 
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', 
	O.id, O.target, 'C', 
	t.id, t.target_table, 'T' 
from dsy.repl_sub_tables t 
where t.id=N.repl_sub_table_id 
; end;


drop trigger DSY.DEL_CFG_RDB_COL;
create trigger DSY.DEL_CFG_RDB_COL 
after delete on DSY.RDB_SUB_COLUMNS 
referencing OLD as O  
for each row mode db2sql     
begin atomic 

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type) 
select 'D', 
	O.id, O.target, 'C', 
	t.id, t.target_table, 'T'   
from dsy.repl_sub_tables t 
where t.id=O.repl_sub_table_id  
; end;


--*rdb_sub_indexes related triggers:

drop trigger DSY.INS_CFG_RDB_INDEX;
create trigger DSY.INS_CFG_RDB_INDEX 
after insert on DSY.RDB_SUB_INDEXES 
referencing NEW as N 
for each row mode db2sql 
begin atomic 

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type) 
select 'A', 
	N.id, N.name, 'I', 
	t.id, t.target_table, 'T'  
from dsy.repl_sub_tables t   
where t.id=N.repl_sub_table_id 
; end; 


drop trigger DSY.UPD_CFG_RDB_INDEX;
create trigger DSY.UPD_CFG_RDB_INDEX 
after update of NAME, COLNAMES on DSY.RDB_SUB_INDEXES 
referencing NEW as N OLD as O 
for each row mode db2sql 
when(not (N.name=O.name and N.colnames=O.colnames)) 
begin atomic 

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type) 
select 'U', 
	O.id, O.name, 'I', 
	t.id, t.target_table, 'T'  
from dsy.repl_sub_tables t   
where t.id=N.repl_sub_table_id 
; end;


drop trigger DSY.DEL_CFG_RDB_INDEX;
create trigger DSY.DEL_CFG_RDB_INDEX 
after delete on DSY.RDB_SUB_INDEXES 
referencing OLD as O 
for each row mode db2sql 
begin atomic 

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type) 
select 'D', 
	O.id, O.name, 'I', 
	t.id, t.target_table, 'T'  
from dsy.repl_sub_tables t 
where t.id=O.repl_sub_table_id 
; end;


--*DATA_LEVEL TRIGGERS DATA_LEVEL TRIGGERS DATA_LEVEL TRIGGERS DATA_LEVEL TRIGGERS
--*DATA_LEVEL TRIGGERS DATA_LEVEL TRIGGERS DATA_LEVEL TRIGGERS DATA_LEVEL TRIGGERS
--*DATA_LEVEL TRIGGERS DATA_LEVEL TRIGGERS DATA_LEVEL TRIGGERS DATA_LEVEL TRIGGERS
--*DATA_LEVEL TRIGGERS DATA_LEVEL TRIGGERS DATA_LEVEL TRIGGERS DATA_LEVEL TRIGGERS
--*DATA_LEVEL TRIGGERS DATA_LEVEL TRIGGERS DATA_LEVEL TRIGGERS DATA_LEVEL TRIGGERS

--*In below triggers, _DL_ stands for DATA_LEVEL.

--*propagate changes to data level of the repl_sub_table:
drop trigger DSY.UPD_DL_TABLES;
create trigger DSY.UPD_DL_TABLES
after update of DATA_LEVEL on DSY.REPL_SUB_TABLES
referencing NEW as N
for each row mode db2sql   
begin atomic
update dsy.subscriptions set data_level=N.data_level where dsy.subscriptions.id=N.subscription_id; end;

--*propagate changes to data level of the subscriptions:
drop trigger DSY.UPD_DL_SUBS;
create trigger DSY.UPD_DL_SUBS
after update of DATA_LEVEL on DSY.SUBSCRIPTIONS
referencing NEW as N
for each row mode db2sql   
begin atomic
update dsy.applications set data_level=N.data_level where dsy.applications.id in
(select application_id from dsy.application_subs where subscription_id=N.id); end;


--*propagate changes to data level of the applications:
drop trigger DSY.UPD_DL_APPS;
create trigger DSY.UPD_DL_APPS
after update of DATA_LEVEL on DSY.APPLICATIONS
referencing NEW as N
for each row mode db2sql   
begin atomic
update dsy.groups set data_level=N.data_level where dsy.groups.id in
(select group_id from dsy.group_applications where application_id=N.id); end;





--*  CACHE LEVEL TRIGGERS CACHE LEVEL TRIGGERS CACHE LEVEL TRIGGERS CACHE LEVEL TRIGGERS
--*  CACHE LEVEL TRIGGERS CACHE LEVEL TRIGGERS CACHE LEVEL TRIGGERS CACHE LEVEL TRIGGERS
--*  CACHE LEVEL TRIGGERS CACHE LEVEL TRIGGERS CACHE LEVEL TRIGGERS CACHE LEVEL TRIGGERS
--*  CACHE LEVEL TRIGGERS CACHE LEVEL TRIGGERS CACHE LEVEL TRIGGERS CACHE LEVEL TRIGGERS


--* UPDATE CACHE ON UPDATE TRIGGERS
--* UPDATE CACHE ON UPDATE TRIGGERS
--* UPDATE CACHE ON UPDATE TRIGGERS
--* UPDATE CACHE ON UPDATE TRIGGERS


--*These are the triggers that are used to track config_changes that are relevant to the server 
--*cache, but that are not relevant to the device config.
--*These changes are not tracked but device config triggers, so in addition to device config triggers, we have these.

--*Note that for subsets and subs, we do not record the correct parent, since there may be
--*multiple parents and we dont care. If we want to access the cached object, we can do so by id,
--*without knowing it's parent. However, for tables, columns, we will record exact parent, because
--*they only have one parent. This will leave the door open to re-load cache in a fine grained way.
--*eg. when a table is changed, we can go to the cache and change that table only.

--*possible udpates to DSY.APPLICATIONS that are not tracked by device config triggers:
drop trigger DSY.UPD_CACHE_APP;
create trigger DSY.UPD_CACHE_APP
after update of DESCRIPTION, SIGNATURE on DSY.APPLICATIONS
referencing NEW as N OLD as O 
for each row mode db2sql  
when( (N.DESCRIPTION is NULL and O.DESCRIPTION is not NULL) or
	  (O.DESCRIPTION is NULL and N.DESCRIPTION is not NULL) or 
	  (not N.DESCRIPTION=O.DESCRIPTION) or
	  (N.SIGNATURE is NULL and O.SIGNATURE is not NULL) or
	  (O.SIGNATURE is NULL and N.SIGNATURE is not NULL) or 
	  (not N.SIGNATURE=O.SIGNATURE))
begin atomic
insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('U', O.id, O.name, 'A', x'00000000000000000000000000', '', 'G'); end;


--*possible udpates to DSY.SUBSCRIPTIONS that are not tracked by device config triggers:
drop trigger DSY.UPD_CACHE_SUBS;
create trigger DSY.UPD_CACHE_SUBS
after update of DESCRIPTION, ADAPTER_ID on DSY.SUBSCRIPTIONS
referencing NEW as N OLD as O  
for each row mode db2sql   
when( (N.DESCRIPTION is NULL and O.DESCRIPTION is not NULL) or
	  (O.DESCRIPTION is NULL and N.DESCRIPTION is not NULL) or 
	  (not N.DESCRIPTION=O.DESCRIPTION) or
	  (N.ADAPTER_ID is NULL and O.ADAPTER_ID is not NULL) or
	  (O.ADAPTER_ID is NULL and N.ADAPTER_ID is not NULL) or 
	  (not N.ADAPTER_ID=O.ADAPTER_ID)) 
begin atomic
insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('U', O.id, O.name, 'S', x'00000000000000000000000000', '', 'A'); end;

--*possible udpates to DSY.REPL_SUB_TABLES that are not tracked by device config triggers:
--*CANCELLED - these updates cause deleting and adding the related row. Not an update.
--*CACELLATION also fixes the 'tablespace page size too small' problem.
drop trigger DSY.UPD_CACHE_TABLES;
--*create trigger DSY.UPD_CACHE_TABLES
--*after update of SOURCE_SCHEMA, SOURCE_TABLE, WHERE_CLAUSE2, 
--*				TABLESPACE, JOINFILTER on DSY.REPL_SUB_TABLES
--*referencing NEW as N OLD as O 
--*for each row mode db2sql   
--*when((N.WHERE_CLAUSE2 is NULL and O.WHERE_CLAUSE2 is not NULL) or
--*	 (O.WHERE_CLAUSE2 is NULL and N.WHERE_CLAUSE2 is not NULL) or  
--*	 (not N.WHERE_CLAUSE2=O.WHERE_CLAUSE2) or
--*	 (N.TABLESPACE is NULL and O.TABLESPACE is not NULL) or
--*	 (O.TABLESPACE is NULL and N.TABLESPACE is not NULL) or  
--*	 (not (N.TABLESPACE=O.TABLESPACE and N.JOINFILTER=O.JOINFILTER and
--*	 	N.SOURCE_SCHEMA=O.SOURCE_SCHEMA and N.SOURCE_TABLE=O.SOURCE_TABLE)) ) 
--*begin atomic
--*
--*insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type) 
--*select 'U', O.id, O.target_table, 'T', x'00000000000000000000000000', s.name, 'S'  
--*from dsy.subscriptions s  
--*where s.id=N.subscription_id 
--*; end;


--*possible udpates to DSY.RDB_SUB_COLUMNS that are not tracked by device config triggers:
drop trigger DSY.UPD_CACHE_COL;
create trigger DSY.UPD_CACHE_COL
after update of SOURCE, REPLICATE, DATAFILTER on DSY.RDB_SUB_COLUMNS
referencing NEW as N OLD as O  
for each row mode db2sql   
when( (N.DATAFILTER is NULL and O.DATAFILTER is not NULL) or
	  (O.DATAFILTER is NULL and N.DATAFILTER is not NULL) or	
	  (not (N.DATAFILTER=O.DATAFILTER and 
	  		N.SOURCE=O.SOURCE and N.REPLICATE=O.REPLICATE) )) 
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', O.id, O.target, 'C', x'00000000000000000000000000', t.target_table, 'T' 
from dsy.repl_sub_tables t 
where t.id=N.repl_sub_table_id 
; end;


--*NOTE:possible udpates to DSY.RDB_SUB_INDEXES are ALREADY tracked by device config triggers.


--*possible udpates to DSY.REPL_SUBSCRIPTIONS that are not tracked by device config triggers:
drop trigger DSY.UPD_CACHE_RS;
create trigger DSY.UPD_CACHE_RS
after update of REPL_MASTER_ID, REPL_MIRROR_ID, TARGET_DATABASE, APPLY_QUALIFIER, MIRROR_ONLY on DSY.REPL_SUBSCRIPTIONS 
referencing NEW as N OLD as O  
for each row mode db2sql   
when( (N.TARGET_DATABASE is NULL and O.TARGET_DATABASE is not NULL) or
	  (O.TARGET_DATABASE is NULL and N.TARGET_DATABASE is not NULL) or	
	  (not (N.TARGET_DATABASE=O.TARGET_DATABASE and 
	  		N.REPL_MASTER_ID=O.REPL_MASTER_ID and N.REPL_MIRROR_ID=O.REPL_MIRROR_ID and
	  		N.APPLY_QUALIFIER=O.APPLY_QUALIFIER and N.MIRROR_ONLY=O.MIRROR_ONLY) )) 
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', s.id, s.name, 'S', x'00000000000000000000000000', s.name, 'S' 
from dsy.subscriptions s 
where s.id=N.subscription_id 
; end;

--*possible udpates to DSY.JDBC_SUBSCRIPTIONS that are not tracked by device config triggers:
drop trigger DSY.UPD_CACHE_JS;
create trigger DSY.UPD_CACHE_JS
after update of JDBC_MASTER_ID, JDBC_MIRROR_ID, TARGET_DATABASE on DSY.JDBC_SUBSCRIPTIONS 
referencing NEW as N OLD as O  
for each row mode db2sql   
when( (N.TARGET_DATABASE is NULL and O.TARGET_DATABASE is not NULL) or
	  (O.TARGET_DATABASE is NULL and N.TARGET_DATABASE is not NULL) or	
	  (not N.TARGET_DATABASE=O.TARGET_DATABASE) or
	  (N.JDBC_MIRROR_ID is NULL and O.JDBC_MIRROR_ID is not NULL) or
	  (O.JDBC_MIRROR_ID is NULL and N.JDBC_MIRROR_ID is not NULL) or	
	  (not N.JDBC_MIRROR_ID=O.JDBC_MIRROR_ID) or
	  (not N.JDBC_MASTER_ID=O.JDBC_MASTER_ID) ) 
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', s.id, s.name, 'S', x'00000000000000000000000000', s.name, 'S' 
from dsy.subscriptions s 
where s.id=N.subscription_id 
; end;


--*possible udpates to DSY.FILE_SUBSCRIPTIONS that are not tracked by device config triggers:
drop trigger DSY.UPD_CACHE_FS;
create trigger DSY.UPD_CACHE_FS
after update of SOURCE, TARGET, DIRTY on DSY.FILE_SUBSCRIPTIONS
referencing NEW as N OLD as O  
for each row mode db2sql   
when((N.TARGET is NULL and O.TARGET is not NULL) or
	  (O.TARGET is NULL and N.TARGET is not NULL) or	
	  (not (N.TARGET=O.TARGET and N.SOURCE=O.SOURCE and N.DIRTY=O.DIRTY) )) 
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', s.id, s.name, 'S', x'00000000000000000000000000', s.name, 'S' 
from dsy.subscriptions s  
where s.id=N.subscription_id
; end;


--*possible udpates to DSY.REPL_MASTER that are not tracked by device config triggers:
drop trigger DSY.UPD_CACHE_RMAST;
create trigger DSY.UPD_CACHE_RMAST
after update of SYSTEM,INSTANCE,DATABASE,USERID,PASSWORD,DATABASEALIAS on DSY.REPL_MASTER 
referencing NEW as N OLD as O  
for each row mode db2sql   
when((N.DATABASEALIAS is NULL and O.DATABASEALIAS is not NULL) or
	  (O.DATABASEALIAS is NULL and N.DATABASEALIAS is not NULL) or	
	  (not (N.DATABASEALIAS=O.DATABASEALIAS and 
	  		N.SYSTEM=O.SYSTEM and N.INSTANCE=O.INSTANCE and
	  		N.DATABASE=O.DATABASE and N.USERID=O.USERID and
	  		N.PASSWORD=O.PASSWORD) )) 
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', rm.id, rm.database, '1', x'00000000000000000000000000', rm.database, '1' 
from dsy.repl_master rm  
where rm.id=N.id
; end;


--*possible udpates to DSY.REPL_MIRROR that are not tracked by device config triggers:
drop trigger DSY.UPD_CACHE_RMIRR;
create trigger DSY.UPD_CACHE_RMIRR
after update of DATABASE,USERID,PASSWORD,SYNC_WINDOW,SYNC_MAX_CLIENTS on DSY.REPL_MIRROR 
referencing NEW as N OLD as O  
for each row mode db2sql   
when( (N.SYNC_WINDOW is NULL and O.SYNC_WINDOW is not NULL) or
	  (O.SYNC_WINDOW is NULL and N.SYNC_WINDOW is not NULL) or
	  (not N.SYNC_WINDOW=O.SYNC_WINDOW)	or
	  (N.SYNC_MAX_CLIENTS is NULL and O.SYNC_MAX_CLIENTS is not NULL) or
	  (O.SYNC_MAX_CLIENTS is NULL and N.SYNC_MAX_CLIENTS is not NULL) or
	  (not N.SYNC_MAX_CLIENTS=O.SYNC_MAX_CLIENTS)	or 
	  (not (N.DATABASE=O.DATABASE and N.USERID=O.USERID and
	  		N.PASSWORD=O.PASSWORD) )) 
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', rmir.id, rmir.database, '2', x'00000000000000000000000000', rmir.database, '2' 
from dsy.repl_mirror rmir  
where rmir.id=N.id
; end;


--*possible udpates to DSY.JDBC_MIRROR that are not tracked by device config triggers:
drop trigger DSY.UPD_CACHE_JMIRR;
create trigger DSY.UPD_CACHE_JMIRR
after update of DATABASE,DRIVER,USERID,PASSWORD,SYNC_WINDOW,SYNC_MAX_CLIENTS on DSY.JDBC_MIRROR 
referencing NEW as N OLD as O  
for each row mode db2sql   
when( (N.SYNC_WINDOW is NULL and O.SYNC_WINDOW is not NULL) or
	  (O.SYNC_WINDOW is NULL and N.SYNC_WINDOW is not NULL) or
	  (not N.SYNC_WINDOW=O.SYNC_WINDOW)	or
	  (N.SYNC_MAX_CLIENTS is NULL and O.SYNC_MAX_CLIENTS is not NULL) or
	  (O.SYNC_MAX_CLIENTS is NULL and N.SYNC_MAX_CLIENTS is not NULL) or
	  (not N.SYNC_MAX_CLIENTS=O.SYNC_MAX_CLIENTS)	or 
	  (not (N.DATABASE=O.DATABASE and N.USERID=O.USERID and
	  		N.PASSWORD=O.PASSWORD and N.DRIVER=O.DRIVER) )) 
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', jmir.id, jmir.database, '3', x'00000000000000000000000000', jmir.database, '3' 
from dsy.jdbc_mirror jmir  
where jmir.id=N.id
; end;


--*possible udpates to DSY.JDBC_MASTER that are not tracked by device config triggers:
drop trigger DSY.UPD_CACHE_JMAST;
create trigger DSY.UPD_CACHE_JMAST
after update of DATABASE,DRIVER,USERID,PASSWORD,CONTROL_TABLESPACE on DSY.JDBC_MASTER 
referencing NEW as N OLD as O  
for each row mode db2sql   
when( (not (N.DATABASE=O.DATABASE and N.USERID=O.USERID and
	  		N.PASSWORD=O.PASSWORD and N.DRIVER=O.DRIVER and 
	  		N.CONTROL_TABLESPACE=O.CONTROL_TABLESPACE)) or 
	  		(O.CONTROL_TABLESPACE is NULL and N.CONTROL_TABLESPACE is not NULL) or
	  		(N.CONTROL_TABLESPACE is NULL and O.CONTROL_TABLESPACE is not NULL)) 
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', jm.id, jm.database, '4', x'00000000000000000000000000', jm.database, '4' 
from dsy.jdbc_master jm  
where jm.id=N.id
; end;


--*possible udpates to DSY.CL_IMPL_MAP that are not tracked by device config triggers:
drop trigger DSY.UPD_CACHE_CLMAP;
create trigger DSY.UPD_CACHE_CLMAP
after update of OBJECT_ID,TYPE,CLASSNAME,ENABLED on DSY.CL_IMPL_MAP 
referencing NEW as N OLD as O  
for each row mode db2sql   
when( (N.CLASSNAME is NULL and O.CLASSNAME is not NULL) or
	  (O.CLASSNAME is NULL and N.CLASSNAME is not NULL) or
	  (not N.CLASSNAME=O.CLASSNAME)	or 
	  (not (N.OBJECT_ID=O.OBJECT_ID and N.TYPE=O.TYPE and
	  		N.ENABLED=O.ENABLED) )) 
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', cl.object_id, CHAR(cl.type), '5', x'00000000000000000000000000', cl.object_id, '5' 
from dsy.cl_impl_map cl  
where cl.object_id=N.object_id and cl.type=N.type
; end;



--*possible udpates to DSY.CL_IMPL_MAP that are not tracked by device config triggers:
drop trigger DSY.UPD_CACHE_CLMAP;
create trigger DSY.UPD_CACHE_CLMAP
after update of OBJECT_ID,TYPE,CLASSNAME,ENABLED on DSY.CL_IMPL_MAP 
referencing NEW as N OLD as O  
for each row mode db2sql   
when( (N.CLASSNAME is NULL and O.CLASSNAME is not NULL) or
	  (O.CLASSNAME is NULL and N.CLASSNAME is not NULL) or
	  (not N.CLASSNAME=O.CLASSNAME)	or 
	  (not (N.OBJECT_ID=O.OBJECT_ID and N.TYPE=O.TYPE and
	  		N.ENABLED=O.ENABLED) )) 
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', cl.object_id, CHAR(cl.type), '5', x'00000000000000000000000000', cl.object_id, '5' 
from dsy.cl_impl_map cl  
where cl.object_id=N.object_id and cl.type=N.type
; end;


--*possible udpates to DSY.ADAPTERS that are not tracked by device config triggers:
drop trigger DSY.UPD_CACHE_ADP;
create trigger DSY.UPD_CACHE_ADP
after update of NAME,DESCRIPTION,FILE_ATTRIBUTES,COMM_ATTRIBUTES,SIGNATURE,CLASSNAME on DSY.ADAPTERS 
referencing NEW as N OLD as O  
for each row mode db2sql   
when( (N.CLASSNAME is NULL and O.CLASSNAME is not NULL) or
	  (O.CLASSNAME is NULL and N.CLASSNAME is not NULL) or
	  (not N.CLASSNAME=O.CLASSNAME)	or 
	  (N.SIGNATURE is NULL and O.SIGNATURE is not NULL) or
	  (O.SIGNATURE is NULL and N.SIGNATURE is not NULL) or
	  (not N.SIGNATURE=O.SIGNATURE)	or 	
	  (N.COMM_ATTRIBUTES is NULL and O.COMM_ATTRIBUTES is not NULL) or
	  (O.COMM_ATTRIBUTES is NULL and N.COMM_ATTRIBUTES is not NULL) or
	  (not N.COMM_ATTRIBUTES=O.COMM_ATTRIBUTES)	or 	
	  (N.FILE_ATTRIBUTES is NULL and O.FILE_ATTRIBUTES is not NULL) or
	  (O.FILE_ATTRIBUTES is NULL and N.FILE_ATTRIBUTES is not NULL) or
	  (not N.FILE_ATTRIBUTES=O.FILE_ATTRIBUTES)	or 	
	  (N.DESCRIPTION is NULL and O.DESCRIPTION is not NULL) or
	  (O.DESCRIPTION is NULL and N.DESCRIPTION is not NULL) or
	  (not N.DESCRIPTION=O.DESCRIPTION)	or 	
	  (not N.NAME=O.NAME) ) 
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
select 'U', adp.id, adp.name, '6', x'00000000000000000000000000', adp.name, '6' 
from dsy.adapters adp  
where adp.id=N.id
; end;


--* UPDATE CACHE ON INSERT TRIGGERS
--* UPDATE CACHE ON INSERT TRIGGERS
--* UPDATE CACHE ON INSERT TRIGGERS
--* UPDATE CACHE ON INSERT TRIGGERS

--*track inserts into dsy.subscriptions, even if the sub is not associated with an app yet.
drop trigger DSY.INS_CACHE_SUBS;
create trigger DSY.INS_CACHE_SUBS
after insert on DSY.SUBSCRIPTIONS
referencing NEW as N  
for each row mode db2sql     
begin atomic
insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('A', N.id, N.name, 'S', x'00000000000000000000000000', '', 'A'); end;

--*track inserts into dsy.applications, even if the app is not associated with a group yet.
drop trigger DSY.INS_CACHE_APPS;
create trigger DSY.INS_CACHE_APPS
after insert on DSY.APPLICATIONS
referencing NEW as N  
for each row mode db2sql     
begin atomic
insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('A', N.id, N.name, 'A', x'00000000000000000000000000', '', 'G'); end;


--*track inserts to DSY.REPL_MASTER for cache validation purposes:
drop trigger DSY.INS_CACHE_RM;
create trigger DSY.INS_CACHE_RM
after insert on DSY.REPL_MASTER
referencing NEW as N  
for each row mode db2sql     
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('A', N.id, N.database, '1', x'00000000000000000000000000', N.database, '1'); end;


--*track inserts to DSY.REPL_MIRROR for cache validation purposes:
drop trigger DSY.INS_CACHE_RMIRR;
create trigger DSY.INS_CACHE_RMIRR
after insert on DSY.REPL_MIRROR
referencing NEW as N  
for each row mode db2sql     
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('A', N.id, N.database, '2', x'00000000000000000000000000', N.database, '2'); end;


--*track inserts to DSY.JDBC_MIRROR for cache validation purposes:
drop trigger DSY.INS_CACHE_JMIRR;
create trigger DSY.INS_CACHE_JMIRR
after insert on DSY.JDBC_MIRROR
referencing NEW as N  
for each row mode db2sql     
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('A', N.id, N.database, '3', x'00000000000000000000000000', N.database, '3'); end;


--*track inserts to DSY.JDBC_MASTER for cache validation purposes:
drop trigger DSY.INS_CACHE_JM;
create trigger DSY.INS_CACHE_JM
after insert on DSY.JDBC_MASTER
referencing NEW as N  
for each row mode db2sql     
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('A', N.id, N.database, '4', x'00000000000000000000000000', N.database, '4'); end;


--*track inserts to DSY.CL_IMPL_MAP for cache validation purposes:
drop trigger DSY.INS_CACHE_CLMAP;
create trigger DSY.INS_CACHE_CLMAP
after insert on DSY.CL_IMPL_MAP
referencing NEW as N  
for each row mode db2sql     
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('A', N.object_id, CHAR(N.type), '5', x'00000000000000000000000000', N.object_id, '5'); end;


--*track inserts to DSY.ADAPTERS for cache validation purposes:
drop trigger DSY.INS_CACHE_ADP;
create trigger DSY.INS_CACHE_ADP
after insert on DSY.ADAPTERS
referencing NEW as N  
for each row mode db2sql     
begin atomic

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('A', N.id, N.name, '6', x'00000000000000000000000000', N.name, '6'); end;


--* UPDATE CACHE ON DELETE TRIGGERS
--* UPDATE CACHE ON DELETE TRIGGERS
--* UPDATE CACHE ON DELETE TRIGGERS
--* UPDATE CACHE ON DELETE TRIGGERS


--*track deletes from DSY.REPL_MASTER for cache validation purposes:
drop trigger DSY.DEL_CACHE_RM;
create trigger DSY.DEL_CACHE_RM 
after delete on DSY.REPL_MASTER 
referencing OLD as O  
for each row mode db2sql     
begin atomic 

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('D', O.id, O.database, '1', x'00000000000000000000000000', O.database, '1'); end;


--*track deletes from DSY.REPL_MIRROR for cache validation purposes:
drop trigger DSY.DEL_CACHE_RMIRR;
create trigger DSY.DEL_CACHE_RMIRR 
after delete on DSY.REPL_MIRROR  
referencing OLD as O  
for each row mode db2sql     
begin atomic 

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('D', O.id, O.database, '2', x'00000000000000000000000000', O.database, '2'); end;


--*track deletes from DSY.JDBC_MIRROR for cache validation purposes:
drop trigger DSY.DEL_CACHE_JMIRR;
create trigger DSY.DEL_CACHE_JMIRR 
after delete on DSY.JDBC_MIRROR  
referencing OLD as O  
for each row mode db2sql     
begin atomic 

delete from dsy.cl_impl_map cl 
where cl.object_id=O.id;--*

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('D', O.id, O.database, '3', x'00000000000000000000000000', O.database, '3'); end;


--*track deletes from DSY.JDBC_MASTER for cache validation purposes:
drop trigger DSY.DEL_CACHE_JM;
create trigger DSY.DEL_CACHE_JM 
after delete on DSY.JDBC_MASTER  
referencing OLD as O  
for each row mode db2sql     
begin atomic 

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('D', O.id, O.database, '4', x'00000000000000000000000000', O.database, '4'); end;


--*track deletes from DSY.CL_IMPL_MAP for cache validation purposes:
drop trigger DSY.DEL_CACHE_CLMAP;
create trigger DSY.DEL_CACHE_CLMAP 
after delete on DSY.CL_IMPL_MAP   
referencing OLD as O  
for each row mode db2sql     
begin atomic 

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('D', O.object_id, CHAR(O.type), '5', x'00000000000000000000000000', O.object_id, '5'); end;


--*track deletes from DSY.ADAPTERS for cache validation purposes:
drop trigger DSY.DEL_CACHE_ADP;
create trigger DSY.DEL_CACHE_ADP 
after delete on DSY.ADAPTERS  
referencing OLD as O  
for each row mode db2sql     
begin atomic 

insert into dsy.config_changes(state,object_id,object_name,object_type,parent_object_id,parent_object_name,parent_object_type)  
values('D', O.id, O.name, '6', x'00000000000000000000000000', O.name, '6'); end;
--* CREATE THE LOCK TABLE AND LOCK_STATUS TABLE FOR MIRROR BLOCKING

create table DSY.LOCK (
    ID char(13) for bit data not null primary key,
    LOCKED INT);

alter table DSY.LOCK VOLATILE;

create table DSY.LOCKSTATUS (
    ID char(13) for bit data not null primary key,
    STATUS INT);

alter table DSY.LOCKSTATUS VOLATILE;

create trigger DSY.LOCK_INSERT_JDBC after insert on DSY.JDBC_MIRROR
referencing new as n for each row mode db2sql begin atomic
insert into DSY.LOCK values (n.ID,0);--*
insert into DSY.LOCKSTATUS values (n.ID,0);--*
end;

create trigger DSY.LOCK_INSERT_DPROPR after insert on DSY.REPL_MIRROR 
referencing new as n for each row mode db2sql begin atomic
insert into DSY.LOCK values (n.ID,0);--*
insert into DSY.LOCKSTATUS values (n.ID,0);--*
end;

create trigger DSY.LOCK_DELETE_JDBC after delete on DSY.JDBC_MIRROR 
referencing old as o for each row mode db2sql begin atomic
delete from DSY.LOCK where  ID = o.ID;--*
delete from DSY.LOCKSTATUS where ID = o.ID;--*
end;

create trigger DSY.LOCK_DELETE_DPROPR after delete on DSY.REPL_MIRROR 
referencing old as o for each row mode db2sql begin atomic
delete from DSY.LOCK where  ID = o.ID;--*
delete from DSY.LOCKSTATUS where id = o.ID;--*
end;

--* Now populate the lock tables from the data from JDBC and DPROPR mirrors
insert into DSY.LOCK (ID,LOCKED) select ID,0 from DSY.JDBC_MIRROR;
insert into DSY.LOCKSTATUS (ID,STATUS) select ID,0 from DSY.JDBC_MIRROR;
insert into DSY.LOCK (ID,LOCKED) select ID,0 from DSY.REPL_MIRROR;
insert into DSY.LOCKSTATUS (ID,STATUS) select ID,0 from DSY.REPL_MIRROR;

--* mlp#9136
delete from dsy.rdb_sub_columns where(repl_sub_table_id not in (select id from dsy.repl_sub_tables));
delete from dsy.rdb_sub_indexes where(repl_sub_table_id not in (select id from dsy.repl_sub_tables));

--* Finished
